package main_test

import (
	"context"
	"crypto/ecdsa"

	//"encoding/hex"
	"math/big"
	"net"
	"testing"
	"time"

	"sync"

	"google.golang.org/grpc"
	"google.golang.org/grpc/credentials/insecure"

	base "github.com/CaduceusMetaverseProtocol/metaprotocol/gen/proto/go/base/v1"
	ring "github.com/CaduceusMetaverseProtocol/metaprotocol/gen/proto/go/ring/v1"
	"github.com/ethereum/go-ethereum/common"
	"github.com/ethereum/go-ethereum/core/types"
	"github.com/ethereum/go-ethereum/crypto"
)

// func BenchmarkFib10(b *testing.B) {
// 	for n := 0; n < b.N; n++ {
// 		//Fib(10)
// 	}
// }

type RingServer struct {
	ring.UnimplementedRingServiceServer
}

func (*RingServer) SendRawTransaction(ctx context.Context, req *base.EthTx) (*ring.SendRawTransactionResponse, error) {

	// txHashAsBytes, err := hex.DecodeString("0xba8e1c42813d24f48b4c785fb893dcf9d9635c660701af1a7f84734f320e8dea")

	// if err != nil {
	// 	return nil, err
	// }

	// _ = txHashAsBytes
	return &ring.SendRawTransactionResponse{}, nil
}

var once sync.Once

func BenchmarkEthTx(b *testing.B) {
	b.ReportAllocs()

	onceFunc := func() {

		lis, err := net.Listen("tcp", ":9006")
		if err != nil {
			b.Fatal(err)
		}

		s := grpc.NewServer()

		ring.RegisterRingServiceServer(s, &RingServer{})

		go func() {
			err = s.Serve(lis)
			if err != nil {
				b.Fatal(err)
			}
		}()
	}

	once.Do(onceFunc)

	b.RunParallel(func(pb *testing.PB) {

		for pb.Next() {

			conn, err := grpc.Dial("127.0.0.1:9006", grpc.WithTransportCredentials(insecure.NewCredentials()))
			if err != nil {
				b.Fatal(err)
			}
			defer conn.Close()

			c := ring.NewRingServiceClient(conn)

			ctx, cancel := context.WithTimeout(context.Background(), time.Second)
			defer cancel()

			local, _ := crypto.HexToECDSA("FD5CC6F5E7E2805E920AC5DC83D5AF1106F9C92F0C04F9D5E1FD4261B4B4464A")

			//publicKey := local.Public()
			//publicKeyECDSA, _ := publicKey.(*ecdsa.PublicKey)
			//fromAddress := crypto.PubkeyToAddress(*publicKeyECDSA)
			//b.Log(fromAddress)

			remote, _ := crypto.GenerateKey()

			tx := pricedTransaction(crypto.PubkeyToAddress(remote.PublicKey), 0, 100000, big.NewInt(1), local)

			inner := base.EthTxData{
				AccountNonce: tx.Nonce(),
				Price:        tx.GasPrice().Bytes(),
				GasLimit:     tx.Gas(),
				Payload:      tx.Data(),
			}

			v, r, sigs := tx.RawSignatureValues()

			inner.V = v.Bytes()
			inner.R = r.Bytes()
			inner.S = sigs.Bytes()
			inner.Amount = tx.Value().Bytes()

			addr := base.Address{Address: tx.To().Bytes()}

			inner.Recipient = &addr

			// Each goroutine has its own bytes.Buffer.

			// The loop body is executed b.N times total across all goroutines.

			res, err := c.SendRawTransaction(ctx, &base.EthTx{Inner: &inner})

			if err != nil {
				b.Fatal(err)
			}

			_ = res

			//b.Logf("Greeting: %v\n", res)

		}
	})
}

func pricedTransaction(to common.Address, nonce uint64, gaslimit uint64, gasprice *big.Int, key *ecdsa.PrivateKey) *types.Transaction {
	tx, _ := types.SignTx(types.NewTransaction(nonce, to, big.NewInt(100), gaslimit, gasprice, nil), types.HomesteadSigner{}, key)
	return tx
}

// go test -bench=.
